@@ -2,14 +2,22 @@
 
 # Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
 
-from . import acquisition, exceptions, models, optim, posteriors, test_functions
+from . import (
+    acquisition,
+    exceptions,
+    models,
+    optim,
+    posteriors,
+    settings,
+    test_functions,
+)
 from .cross_validation import batch_cross_validation
 from .fit import fit_gpytorch_model
 from .gen import gen_candidates_scipy, gen_candidates_torch, get_best_candidates
 from .utils import manual_seed
 
 
-__version__ = "0.1.1"
+__version__ = "0.1.4"
 
 
 __all__ = [
@@ -24,5 +32,6 @@ __all__ = [
     "models",
     "optim",
     "posteriors",
+    "settings",
     "test_functions",
 ]
