@@ -2,7 +2,7 @@
 
 # Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
 
-from .acquisition import AcquisitionFunction
+from .acquisition import AcquisitionFunction, OneShotAcquisitionFunction
 from .analytic import (
     AnalyticAcquisitionFunction,
     ConstrainedExpectedImprovement,
@@ -12,6 +12,8 @@ from .analytic import (
     ProbabilityOfImprovement,
     UpperConfidenceBound,
 )
+from .fixed_feature import FixedFeatureAcquisitionFunction
+from .knowledge_gradient import qKnowledgeGradient
 from .monte_carlo import (
     MCAcquisitionFunction,
     qExpectedImprovement,
@@ -26,6 +28,7 @@ from .objective import (
     IdentityMCObjective,
     LinearMCObjective,
     MCAcquisitionObjective,
+    ScalarizedObjective,
 )
 from .utils import get_acquisition_function
 
@@ -35,11 +38,14 @@ __all__ = [
     "AnalyticAcquisitionFunction",
     "ConstrainedExpectedImprovement",
     "ExpectedImprovement",
+    "FixedFeatureAcquisitionFunction",
     "NoisyExpectedImprovement",
+    "OneShotAcquisitionFunction",
     "PosteriorMean",
     "ProbabilityOfImprovement",
     "UpperConfidenceBound",
     "qExpectedImprovement",
+    "qKnowledgeGradient",
     "qNoisyExpectedImprovement",
     "qProbabilityOfImprovement",
     "qSimpleRegret",
@@ -50,5 +56,6 @@ __all__ = [
     "LinearMCObjective",
     "MCAcquisitionFunction",
     "MCAcquisitionObjective",
+    "ScalarizedObjective",
     "get_acquisition_function",
 ]
