@@ -23,21 +23,54 @@ from ..posteriors.posterior import Posterior
 from ..sampling.samplers import SobolQMCNormalSampler
 from ..utils.transforms import convert_to_target_pre_hook, t_batch_mode_transform
 from .acquisition import AcquisitionFunction
+from .objective import ScalarizedObjective
 
 
 class AnalyticAcquisitionFunction(AcquisitionFunction, ABC):
     r"""Base class for analytic acquisition functions."""
 
-    def _validate_single_output_posterior(self, posterior: Posterior) -> None:
-        r"""Validates that the computed posterior is single-output.
+    def __init__(
+        self, model: Model, objective: Optional[ScalarizedObjective] = None
+    ) -> None:
+        r"""Base constructor for analytic acquisition functions.
 
-        Raises an UnsupportedError if not.
+        Args:
+            model: A fitted single-outcome model.
+            objective: A ScalarizedObjective (optional).
         """
-        if posterior.event_shape[-1] != 1:
+        super().__init__(model=model)
+        if objective is not None and not isinstance(objective, ScalarizedObjective):
             raise UnsupportedError(
-                "Multi-Output posteriors are not supported for acquisition "
-                f" function of type {self.__class__.__name__}"
+                "Only objectives of type ScalarizedObjective are supported for "
+                "analytic acquisition functions."
             )
+        self.objective = objective
+
+    def _get_posterior(self, X: Tensor, check_single_output: bool = True) -> Posterior:
+        r"""Compute the posterior at the input candidate set X.
+
+        Applies the objective if provided.
+
+        Args:
+            X: The input candidate set
+            check_single_output: If True, Raise an error if the posterior is not
+                single-output.
+
+        Returns:
+            The posterior at X.
+        """
+        posterior = self.model.posterior(X)
+        if self.objective is not None:
+            # Unlike MCAcquisitionObjectives (which transform samples), this
+            # transforms the posterior
+            posterior = self.objective(posterior)
+        if check_single_output:
+            if posterior.event_shape[-1] != 1:
+                raise UnsupportedError(
+                    "Multi-Output posteriors are not supported for acquisition "
+                    f"functions of type {self.__class__.__name__}"
+                )
+        return posterior
 
     def set_X_pending(self, X_pending: Optional[Tensor] = None) -> None:
         raise UnsupportedError(
@@ -64,7 +97,11 @@ class ExpectedImprovement(AnalyticAcquisitionFunction):
     """
 
     def __init__(
-        self, model: Model, best_f: Union[float, Tensor], maximize: bool = True
+        self,
+        model: Model,
+        best_f: Union[float, Tensor],
+        objective: Optional[ScalarizedObjective] = None,
+        maximize: bool = True,
     ) -> None:
         r"""Single-outcome Expected Improvement (analytic).
 
@@ -72,9 +109,10 @@ class ExpectedImprovement(AnalyticAcquisitionFunction):
             model: A fitted single-outcome model.
             best_f: Either a scalar or a `b`-dim Tensor (batch mode) representing
                 the best function value observed so far (assumed noiseless).
+            objective: A ScalarizedObjective (optional).
             maximize: If True, consider the problem a maximization problem.
         """
-        super().__init__(model=model)
+        super().__init__(model=model, objective=objective)
         self.maximize = maximize
         if not torch.is_tensor(best_f):
             best_f = torch.tensor(best_f)
@@ -95,8 +133,7 @@ class ExpectedImprovement(AnalyticAcquisitionFunction):
             given design points `X`.
         """
         self.best_f = self.best_f.to(X)
-        posterior = self.model.posterior(X)
-        self._validate_single_output_posterior(posterior)
+        posterior = self._get_posterior(X=X)
         mean = posterior.mean
         # deal with batch evaluation and broadcasting
         view_shape = mean.shape[:-2] if mean.dim() >= X.dim() else X.shape[:-2]
@@ -136,8 +173,7 @@ class PosteriorMean(AnalyticAcquisitionFunction):
             A `(b)`-dim Tensor of Posterior Mean values at the given design
             points `X`.
         """
-        posterior = self.model.posterior(X)
-        self._validate_single_output_posterior(posterior)
+        posterior = self._get_posterior(X=X)
         return posterior.mean.view(X.shape[:-2])
 
 
@@ -158,7 +194,11 @@ class ProbabilityOfImprovement(AnalyticAcquisitionFunction):
     """
 
     def __init__(
-        self, model: Model, best_f: Union[float, Tensor], maximize: bool = True
+        self,
+        model: Model,
+        best_f: Union[float, Tensor],
+        objective: Optional[ScalarizedObjective] = None,
+        maximize: bool = True,
     ) -> None:
         r"""Single-outcome analytic Probability of Improvement.
 
@@ -166,9 +206,10 @@ class ProbabilityOfImprovement(AnalyticAcquisitionFunction):
             model: A fitted single-outcome model.
             best_f: Either a scalar or a `b`-dim Tensor (batch mode) representing
                 the best function value observed so far (assumed noiseless).
+            objective: A ScalarizedObjective (optional).
             maximize: If True, consider the problem a maximization problem.
         """
-        super().__init__(model=model)
+        super().__init__(model=model, objective=objective)
         self.maximize = maximize
         if not torch.is_tensor(best_f):
             best_f = torch.tensor(best_f)
@@ -187,10 +228,9 @@ class ProbabilityOfImprovement(AnalyticAcquisitionFunction):
             design points `X`.
         """
         self.best_f = self.best_f.to(X)
-        batch_shape = X.shape[:-2]
-        posterior = self.model.posterior(X)
-        self._validate_single_output_posterior(posterior)
+        posterior = self._get_posterior(X=X)
         mean, sigma = posterior.mean, posterior.variance.sqrt()
+        batch_shape = X.shape[:-2]
         mean = posterior.mean.view(batch_shape)
         sigma = posterior.variance.sqrt().clamp_min(1e-9).view(batch_shape)
         u = (mean - self.best_f.expand_as(mean)) / sigma
@@ -218,7 +258,11 @@ class UpperConfidenceBound(AnalyticAcquisitionFunction):
     """
 
     def __init__(
-        self, model: Model, beta: Union[float, Tensor], maximize: bool = True
+        self,
+        model: Model,
+        beta: Union[float, Tensor],
+        objective: Optional[ScalarizedObjective] = None,
+        maximize: bool = True,
     ) -> None:
         r"""Single-outcome Upper Confidence Bound.
 
@@ -227,9 +271,10 @@ class UpperConfidenceBound(AnalyticAcquisitionFunction):
                 candidate sets X will be)
             beta: Either a scalar or a one-dim tensor with `b` elements (batch mode)
                 representing the trade-off parameter between mean and covariance
+            objective: A ScalarizedObjective (optional).
             maximize: If True, consider the problem a maximization problem.
         """
-        super().__init__(model=model)
+        super().__init__(model=model, objective=objective)
         self.maximize = maximize
         if not torch.is_tensor(beta):
             beta = torch.tensor(beta)
@@ -248,9 +293,8 @@ class UpperConfidenceBound(AnalyticAcquisitionFunction):
             design points `X`.
         """
         self.beta = self.beta.to(X)
+        posterior = self._get_posterior(X=X)
         batch_shape = X.shape[:-2]
-        posterior = self.model.posterior(X)
-        self._validate_single_output_posterior(posterior)
         mean = posterior.mean.view(batch_shape)
         variance = posterior.variance.view(batch_shape)
         delta = (self.beta.expand_as(mean) * variance).sqrt()
@@ -323,9 +367,9 @@ class ConstrainedExpectedImprovement(AnalyticAcquisitionFunction):
             A `(b)`-dim Tensor of Expected Improvement values at the given
             design points `X`.
         """
-        posterior = self.model.posterior(X)
-        means = posterior.mean.squeeze(dim=-2)  # (b) x t
-        sigmas = posterior.variance.squeeze(dim=-2).sqrt().clamp_min(1e-9)  # (b) x t
+        posterior = self._get_posterior(X=X, check_single_output=False)
+        means = posterior.mean.squeeze(dim=-2)  # (b) x m
+        sigmas = posterior.variance.squeeze(dim=-2).sqrt().clamp_min(1e-9)  # (b) x m
 
         # (b) x 1
         mean_obj = means[..., [self.objective_index]]
@@ -392,10 +436,10 @@ class ConstrainedExpectedImprovement(AnalyticAcquisitionFunction):
         Args:
             X: A `(b) x 1 x d`-dim Tensor of `(b)` t-batches of `d`-dim design
                 points each.
-            means: A `(b) x t`-dim Tensor of means.
-            sigmas: A `(b) x t`-dim Tensor of standard deviations.
+            means: A `(b) x m`-dim Tensor of means.
+            sigmas: A `(b) x m`-dim Tensor of standard deviations.
         Returns:
-            A `(b) x 1 x 1`-dim tensor of feasibility probabilities
+            A `(b) x 1`-dim tensor of feasibility probabilities
 
         Note: This function does case-work for upper bound, lower bound, and both-sided
         bounds. Another way to do it would be to use 'inf' and -'inf' for the
@@ -403,8 +447,7 @@ class ConstrainedExpectedImprovement(AnalyticAcquisitionFunction):
         causes an issue with autograd since we get 0 * inf.
         TODO: Investigate further.
         """
-        output_shape = list(X.shape)
-        output_shape[-1] = 1
+        output_shape = X.shape[:-2] + torch.Size([1])
         prob_feas = torch.ones(output_shape, device=X.device, dtype=X.dtype)
 
         if len(self.constraint_lower_inds) > 0:
@@ -454,7 +497,7 @@ class NoisyExpectedImprovement(ExpectedImprovement):
 
         Args:
             model: A fitted single-outcome model.
-            X_observed: A `m x d` Tensor of observed points that are likely to
+            X_observed: A `n x d` Tensor of observed points that are likely to
                 be the best observed points so far.
             num_fantasies: The number of fantasies to generate. The higher this
                 number the more accurate the model (at the expense of model
@@ -467,8 +510,7 @@ class NoisyExpectedImprovement(ExpectedImprovement):
             )
         # sample fantasies
         with torch.no_grad():
-            posterior = model.posterior(X_observed)
-            self._validate_single_output_posterior(posterior=posterior)
+            posterior = model.posterior(X=X_observed)
             sampler = SobolQMCNormalSampler(num_fantasies)
             Y_fantasized = sampler(posterior).squeeze(-1)
         batch_X_observed = X_observed.expand(num_fantasies, *X_observed.shape)
@@ -514,9 +556,9 @@ def _get_noiseless_fantasy_model(
 
     Args:
         model: a fitted FixedNoiseGP
-        batch_X_observed: A `b x m x d` tensor of inputs where `b` is the number of
+        batch_X_observed: A `b x n x d` tensor of inputs where `b` is the number of
             fantasies.
-        Y_fantasized: A `b x m` tensor of fantasized targets where `b` is the number of
+        Y_fantasized: A `b x n` tensor of fantasized targets where `b` is the number of
             fantasies.
 
     Returns:
@@ -528,8 +570,8 @@ def _get_noiseless_fantasy_model(
     # uses independent hyperparameters for each batch).
     fantasy_model = FixedNoiseGP(
         train_X=model.train_inputs[0],
-        train_Y=model.train_targets,
-        train_Yvar=model.likelihood.noise_covar.noise,
+        train_Y=model.train_targets.unsqueeze(-1),
+        train_Yvar=model.likelihood.noise_covar.noise.unsqueeze(-1),
     )
     # update training inputs/targets to be batch mode fantasies
     fantasy_model.set_train_data(
