@@ -14,19 +14,68 @@ from botorch.utils import apply_constraints
 from torch import Tensor
 from torch.nn import Module
 
+from ..posteriors.gpytorch import GPyTorchPosterior, scalarize_posterior
 
-class MCAcquisitionObjective(Module, ABC):
-    r"""Abstract base class for MC-based objectives."""
 
-    def __init__(self) -> None:
+class AcquisitionObjective(Module, ABC):
+    r"""Abstract base class for objectives."""
+
+    ...
+
+
+class ScalarizedObjective(AcquisitionObjective):
+    r"""Affine objective to be used with analytic acquisition functions.
+
+    For a Gaussian posterior at a single point (`q=1`) with mean `mu` and
+    covariance matrix `Sigma`, this yields a single-output posterior with mean
+    `weights^T * mu` and variance `weights^T Sigma w`.
+
+    Example:
+        Example for a model with two outcomes:
+
+        >>> weights = torch.tensor([0.5, 0.25])
+        >>> objective = ScalarizedObjective(weights)
+        >>> EI = ExpectedImprovement(model, best_f=0.1, objective=objective)
+    """
+
+    def __init__(self, weights: Tensor, offset: float = 0.0) -> None:
+        r"""Affine objective.
+
+        Args:
+            weights: A one-dimensional tensor with `m` elements representing the
+                linear weights on the outputs.
+            offset: An offset to be added to posterior mean.
+        """
+        if weights.dim() != 1:
+            raise ValueError("weights must be a one-dimensional tensor.")
         super().__init__()
+        self.register_buffer("weights", weights)
+        self.offset = offset
+
+    def forward(self, posterior: GPyTorchPosterior) -> GPyTorchPosterior:
+        r"""Compute the posterior of the affine transformation.
+
+        Args:
+            posterior: A posterior with the same number of outputs as the
+                elements in `self.weights`.
+
+        Returns:
+            A single-output posterior.
+        """
+        return scalarize_posterior(
+            posterior=posterior, weights=self.weights, offset=self.offset
+        )
+
+
+class MCAcquisitionObjective(AcquisitionObjective):
+    r"""Abstract base class for MC-based objectives."""
 
     @abstractmethod
     def forward(self, samples: Tensor) -> Tensor:
         r"""Evaluate the objective on the samples.
 
         Args:
-            samples: A `sample_shape x batch_shape x q x o`-dim Tensors of
+            samples: A `sample_shape x batch_shape x q x m`-dim Tensors of
                 samples from a model posterior.
 
         Returns:
@@ -75,7 +124,7 @@ class LinearMCObjective(MCAcquisitionObjective):
         r"""Linear Objective.
 
         Args:
-            weights: A one-dimensional tensor with `o` elements representing the
+            weights: A one-dimensional tensor with `m` elements representing the
                 linear weights on the outputs.
         """
         super().__init__()
@@ -87,7 +136,7 @@ class LinearMCObjective(MCAcquisitionObjective):
         r"""Evaluate the linear objective on the samples.
 
         Args:
-            samples: A `sample_shape x batch_shape x q x o`-dim tensors of
+            samples: A `sample_shape x batch_shape x q x m`-dim tensors of
                 samples from a model posterior.
 
         Returns:
@@ -115,7 +164,7 @@ class GenericMCObjective(MCAcquisitionObjective):
         r"""Objective generated from a generic callable.
 
         Args:
-            objective: A callable mapping a `sample_shape x batch-shape x q x o`-
+            objective: A callable mapping a `sample_shape x batch-shape x q x m`-
                 dim Tensor to a `sample_shape x batch-shape x q`-dim Tensor of
                 objective values.
         """
@@ -126,7 +175,7 @@ class GenericMCObjective(MCAcquisitionObjective):
         r"""Evaluate the feasibility-weigthed objective on the samples.
 
         Args:
-            samples: A `sample_shape x batch_shape x q x o`-dim Tensors of
+            samples: A `sample_shape x batch_shape x q x m`-dim Tensors of
                 samples from a model posterior.
 
         Returns:
@@ -169,11 +218,11 @@ class ConstrainedMCObjective(GenericMCObjective):
         r"""Feasibility-weighted objective.
 
         Args:
-            objective: A callable mapping a `sample_shape x batch-shape x q x o`-
+            objective: A callable mapping a `sample_shape x batch-shape x q x m`-
                 dim Tensor to a `sample_shape x batch-shape x q`-dim Tensor of
                 objective values.
             constraints: A list of callables, each mapping a Tensor of dimension
-                `sample_shape x batch-shape x q x o` to a Tensor of dimension
+                `sample_shape x batch-shape x q x m` to a Tensor of dimension
                 `sample_shape x batch-shape x q`, where negative values imply
                 feasibility.
             infeasible_cost: The cost of a design if all associated samples are
@@ -190,7 +239,7 @@ class ConstrainedMCObjective(GenericMCObjective):
         r"""Evaluate the feasibility-weighted objective on the samples.
 
         Args:
-            samples: A `sample_shape x batch_shape x q x o`-dim Tensors of
+            samples: A `sample_shape x batch_shape x q x m`-dim Tensors of
                 samples from a model posterior.
 
         Returns:
