@@ -2,21 +2,34 @@
 
 # Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
 
-from .errors import BotorchError, CandidateGenerationError, UnsupportedError
+from .errors import (
+    BotorchError,
+    BotorchTensorDimensionError,
+    CandidateGenerationError,
+    InputDataError,
+    UnsupportedError,
+)
 from .warnings import (
     BadInitialCandidatesWarning,
+    BotorchTensorDimensionWarning,
     BotorchWarning,
+    InputDataWarning,
     OptimizationWarning,
     SamplingWarning,
 )
 
 
 __all__ = [
-    "BotorchError",
-    "CandidateGenerationError",
-    "UnsupportedError",
-    "BotorchWarning",
     "BadInitialCandidatesWarning",
+    "BotorchError",
+    "BotorchTensorDimensionError",
+    "BotorchTensorDimensionWarning",
+    "BotorchWarning",
+    "InputDataWarning",
+    "InputDataError",
+    "BadInitialCandidatesWarning",
+    "CandidateGenerationError",
     "OptimizationWarning",
     "SamplingWarning",
+    "UnsupportedError",
 ]
