@@ -19,7 +19,19 @@ class CandidateGenerationError(BotorchError):
     pass
 
 
+class InputDataError(BotorchError):
+    r"""Exception raised when input data does not comply with conventions."""
+
+    pass
+
+
 class UnsupportedError(BotorchError):
     r"""Currently unsupported feature."""
 
     pass
+
+
+class BotorchTensorDimensionError(BotorchError):
+    r"""Exception raised when a tensor violates a botorch convention."""
+
+    pass
