@@ -13,19 +13,31 @@ class BotorchWarning(Warning):
     pass
 
 
-class OptimizationWarning(BotorchWarning):
-    r"""Optimization-releated warnings."""
-
-    pass
-
-
 class BadInitialCandidatesWarning(BotorchWarning):
     r"""Warning issued if set of initial candidates for optimziation is bad."""
 
     pass
 
 
+class InputDataWarning(BotorchWarning):
+    r"""Warning raised when input data does not comply with conventions."""
+
+    pass
+
+
+class OptimizationWarning(BotorchWarning):
+    r"""Optimization-releated warnings."""
+
+    pass
+
+
 class SamplingWarning(BotorchWarning):
-    r"""Sampling releated warnings."""
+    r"""Sampling related warnings."""
+
+    pass
+
+
+class BotorchTensorDimensionWarning(BotorchWarning):
+    r"""Warning raised when a tensor possibly violates a botorch convention."""
 
     pass
