@@ -4,13 +4,19 @@
 
 from .initializers import initialize_q_batch, initialize_q_batch_nonneg
 from .numpy_converter import module_to_array, set_params_with_array
-from .optimize import gen_batch_initial_conditions, joint_optimize, sequential_optimize
+from .optimize import (
+    gen_batch_initial_conditions,
+    joint_optimize,
+    optimize_acqf,
+    sequential_optimize,
+)
 
 
 __all__ = [
     "gen_batch_initial_conditions",
     "initialize_q_batch",
     "initialize_q_batch_nonneg",
+    "optimize_acqf",
     "joint_optimize",
     "module_to_array",
     "sequential_optimize",
