@@ -17,6 +17,7 @@ from gpytorch.mlls.sum_marginal_log_likelihood import SumMarginalLogLikelihood
 from gpytorch.mlls.variational_elbo import VariationalELBO
 from torch import Tensor
 
+from ..exceptions.errors import BotorchError
 from ..exceptions.warnings import BotorchWarning
 from ..models.gpytorch import GPyTorchModel
 
@@ -27,13 +28,13 @@ def sample_all_priors(model: GPyTorchModel) -> None:
     Args:
         model: A GPyTorchModel.
     """
-    for _, prior, _, setting_closure in model.named_priors():
+    for _, prior, closure, setting_closure in model.named_priors():
         if setting_closure is None:
             raise RuntimeError(
                 "Must provide inverse transform to be able to sample from prior."
             )
         try:
-            setting_closure(prior.sample())
+            setting_closure(prior.sample(closure().shape))
         except NotImplementedError:
             warnings.warn(
                 f"`rsample` not implemented for {type(prior)}. Skipping.",
@@ -41,30 +42,57 @@ def sample_all_priors(model: GPyTorchModel) -> None:
             )
 
 
-def check_convergence(
-    loss_trajectory: List[float],
-    param_trajectory: Dict[str, List[Tensor]],
-    options: Dict[str, Union[float, str]],
-) -> bool:
-    r"""Check convergence of optimization for pytorch optimizers.
-
-    Right now this is just a dummy function and only checks for maxiter.
-
-    Args:
-        loss_trajectory: A list containing the loss value at each iteration.
-        param_trajectory: A dictionary mapping each parameter name to a list of Tensors
-            where the `i`th Tensor is the parameter value at iteration `i`.
-        options: dictionary of options. Currently only "maxiter" is supported.
-
-    Returns:
-        A boolean indicating whether optimization has converged.
+class ConvergenceCriterion:
+    r"""Basic class for evaluating optimization convergence.
     """
-    maxiter: int = options.get("maxiter", 50)
-    # TODO: Be A LOT smarter about this
-    # TODO: Make this work in batch mode (see parallel L-BFGS-P)
-    if len(loss_trajectory) >= maxiter:
-        return True
-    else:
+
+    def __init__(
+        self,
+        maxiter: int = 15000,
+        ftol: float = 2.220446049250313e-09,
+        minimize: bool = True,
+    ) -> None:
+        r"""Constructor for ConvergenceCriterion.
+
+        Args:
+            maxiter: maximum number of iterations.
+            ftol: Function value relative tolerance for termination.
+            minimize: boolean indicating the optimization direction.
+        """
+        # by default, use the scipy defaults
+        self.maxiter = maxiter
+        self.ftol = ftol
+        self.minimize = minimize
+        self.prev_fvals = None
+        self.iter = 0
+
+    def evaluate(self, fvals: Tensor) -> bool:
+        r"""Evaluate convergence criterion.
+
+        Args:
+            fvals: tensor containing function values for the current iteration.
+                If `fvals` contains more than one element, then the relative
+                tolerance criterion is evaluated element-wise and True is returned
+                if all elements have converged.
+
+        TODO: add support for utilizing gradient information
+
+        Returns:
+            bool: convergence indicator
+        """
+        self.iter += 1
+        if self.iter == self.maxiter:
+            return True
+        elif self.prev_fvals is not None:
+            fmax = torch.stack(
+                [self.prev_fvals.abs(), fvals.abs(), torch.ones_like(fvals)], dim=0
+            ).max(dim=0)[0]
+            f_delta = (self.prev_fvals - fvals).div(fmax)
+            if not self.minimize:
+                f_delta *= -1
+            if torch.all(f_delta <= self.ftol):
+                return True
+        self.prev_fvals = fvals
         return False
 
 
@@ -72,17 +100,24 @@ def columnwise_clamp(
     X: Tensor,
     lower: Optional[Union[float, Tensor]] = None,
     upper: Optional[Union[float, Tensor]] = None,
+    raise_on_violation: bool = False,
 ) -> Tensor:
     r"""Clamp values of a Tensor in column-wise fashion (with support for t-batches).
 
     This function is useful in conjunction with optimizers from the torch.optim
     package, which don't natively handle constraints. If you apply this after
     a gradient step you can be fancy and call it "projected gradient descent".
+    This funtion is also useful for post-processing candidates generated by the
+    scipy optimizer that satisfy bounds only up to numerical accuracy.
 
     Args:
         X: The `b x n x d` input tensor. If 2-dimensional, `b` is assumed to be 1.
         lower: The column-wise lower bounds. If scalar, apply bound to all columns.
         upper: The column-wise upper bounds. If scalar, apply bound to all columns.
+        raise_on_violation: If `True`, raise an exception when the elments in `X`
+            are out of the specified bounds (up to numerical accuracy). This is
+            useful for post-processing candidates generated by optimizers that
+            satisfy imposed bounds only up to numerical accuracy.
 
     Returns:
         The clamped tensor.
@@ -97,6 +132,8 @@ def columnwise_clamp(
         Xout = Xout.max(min_bounds)
     if max_bounds is not None:
         Xout = Xout.min(max_bounds)
+    if raise_on_violation and not torch.allclose(Xout, X):
+        raise BotorchError("Original value(s) are out of bounds.")
     return Xout
 
 
