@@ -3,7 +3,6 @@
 # Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
 
 import math
-import unittest
 
 import torch
 from botorch.acquisition.analytic import (
@@ -15,251 +14,300 @@ from botorch.acquisition.analytic import (
     ProbabilityOfImprovement,
     UpperConfidenceBound,
 )
+from botorch.acquisition.objective import IdentityMCObjective, ScalarizedObjective
 from botorch.exceptions import UnsupportedError
 from botorch.models import FixedNoiseGP, SingleTaskGP
-from botorch.utils.mock import MockModel, MockPosterior
+from botorch.posteriors import GPyTorchPosterior
+from botorch.utils.testing import BotorchTestCase, MockModel, MockPosterior
+from gpytorch.distributions import MultitaskMultivariateNormal, MultivariateNormal
 
 
-NEI_NOISE = [-0.099, -0.004, 0.227, -0.182, 0.018, 0.334, -0.270, 0.156, -0.237, 0.052]
+NEI_NOISE = [
+    [-0.099],
+    [-0.004],
+    [0.227],
+    [-0.182],
+    [0.018],
+    [0.334],
+    [-0.270],
+    [0.156],
+    [-0.237],
+    [0.052],
+]
 
 
-class TestAnalyticAcquisitionFunction(unittest.TestCase):
+class TestAnalyticAcquisitionFunction(BotorchTestCase):
     def test_abstract_raises(self):
         with self.assertRaises(TypeError):
             AnalyticAcquisitionFunction()
 
 
-class TestExpectedImprovement(unittest.TestCase):
-    def test_expected_improvement(self, cuda=False):
-        device = torch.device("cuda") if cuda else torch.device("cpu")
+class TestExpectedImprovement(BotorchTestCase):
+    def test_expected_improvement(self):
         for dtype in (torch.float, torch.double):
-            mean = torch.tensor([[-0.5]], device=device, dtype=dtype)
-            variance = torch.ones(1, 1, device=device, dtype=dtype)
+            mean = torch.tensor([[-0.5]], device=self.device, dtype=dtype)
+            variance = torch.ones(1, 1, device=self.device, dtype=dtype)
             mm = MockModel(MockPosterior(mean=mean, variance=variance))
 
+            # basic test
             module = ExpectedImprovement(model=mm, best_f=0.0)
-            X = torch.empty(1, 1, device=device, dtype=dtype)  # dummy
+            X = torch.empty(1, 1, device=self.device, dtype=dtype)  # dummy
             ei = module(X)
-            ei_expected = torch.tensor(0.19780, device=device, dtype=dtype)
+            ei_expected = torch.tensor(0.19780, device=self.device, dtype=dtype)
             self.assertTrue(torch.allclose(ei, ei_expected, atol=1e-4))
 
+            # test maximize
             module = ExpectedImprovement(model=mm, best_f=0.0, maximize=False)
-            X = torch.empty(1, 1, device=device, dtype=dtype)  # dummy
+            X = torch.empty(1, 1, device=self.device, dtype=dtype)  # dummy
             ei = module(X)
-            ei_expected = torch.tensor(0.6978, device=device, dtype=dtype)
+            ei_expected = torch.tensor(0.6978, device=self.device, dtype=dtype)
             self.assertTrue(torch.allclose(ei, ei_expected, atol=1e-4))
             with self.assertRaises(UnsupportedError):
                 module.set_X_pending(None)
 
-    def test_expected_improvement_cuda(self, cuda=False):
-        if torch.cuda.is_available():
-            self.test_expected_improvement(cuda=True)
+            # test objective (single-output)
+            mean = torch.tensor([0.5], device=self.device, dtype=dtype)
+            covar = torch.tensor([[0.16]], device=self.device, dtype=dtype)
+            mvn = MultivariateNormal(mean, covar)
+            p = GPyTorchPosterior(mvn)
+            mm = MockModel(p)
+            weights = torch.tensor([0.5], device=self.device, dtype=dtype)
+            obj = ScalarizedObjective(weights)
+            ei = ExpectedImprovement(model=mm, best_f=0.0, objective=obj)
+            X = torch.rand(1, 2, device=self.device, dtype=dtype)
+            ei_expected = torch.tensor(0.2601, device=self.device, dtype=dtype)
+            torch.allclose(ei(X), ei_expected, atol=1e-4)
 
-    def test_expected_improvement_batch(self, cuda=False):
-        device = torch.device("cuda") if cuda else torch.device("cpu")
+            # test objective (multi-output)
+            mean = torch.tensor([[-0.25, 0.5]], device=self.device, dtype=dtype)
+            covar = torch.tensor(
+                [[[0.5, 0.125], [0.125, 0.5]]], device=self.device, dtype=dtype
+            )
+            mvn = MultitaskMultivariateNormal(mean, covar)
+            p = GPyTorchPosterior(mvn)
+            mm = MockModel(p)
+            weights = torch.tensor([2.0, 1.0], device=self.device, dtype=dtype)
+            obj = ScalarizedObjective(weights)
+            ei = ExpectedImprovement(model=mm, best_f=0.0, objective=obj)
+            X = torch.rand(1, 2, device=self.device, dtype=dtype)
+            ei_expected = torch.tensor(0.6910, device=self.device, dtype=dtype)
+            torch.allclose(ei(X), ei_expected, atol=1e-4)
+
+    def test_expected_improvement_batch(self):
         for dtype in (torch.float, torch.double):
-            mean = torch.tensor([-0.5, 0.0, 0.5], device=device, dtype=dtype).view(
+            mean = torch.tensor([-0.5, 0.0, 0.5], device=self.device, dtype=dtype).view(
                 3, 1, 1
             )
-            variance = torch.ones(3, 1, 1, device=device, dtype=dtype)
+            variance = torch.ones(3, 1, 1, device=self.device, dtype=dtype)
             mm = MockModel(MockPosterior(mean=mean, variance=variance))
             module = ExpectedImprovement(model=mm, best_f=0.0)
-            X = torch.empty(3, 1, 1, device=device, dtype=dtype)  # dummy
+            X = torch.empty(3, 1, 1, device=self.device, dtype=dtype)  # dummy
             ei = module(X)
             ei_expected = torch.tensor(
-                [0.19780, 0.39894, 0.69780], device=device, dtype=dtype
+                [0.19780, 0.39894, 0.69780], device=self.device, dtype=dtype
             )
             self.assertTrue(torch.allclose(ei, ei_expected, atol=1e-4))
             # check for proper error if multi-output model
-            mean2 = torch.rand(3, 1, 2, device=device, dtype=dtype)
-            variance2 = torch.rand(3, 1, 2, device=device, dtype=dtype)
+            mean2 = torch.rand(3, 1, 2, device=self.device, dtype=dtype)
+            variance2 = torch.rand(3, 1, 2, device=self.device, dtype=dtype)
             mm2 = MockModel(MockPosterior(mean=mean2, variance=variance2))
             module2 = ExpectedImprovement(model=mm2, best_f=0.0)
             with self.assertRaises(UnsupportedError):
                 module2(X)
 
-    def test_expected_improvement_batch_cuda(self, cuda=False):
-        if torch.cuda.is_available():
-            self.test_expected_improvement_batch(cuda=True)
+            # test objective (single-output)
+            mean = torch.tensor([[[0.5]], [[0.25]]], device=self.device, dtype=dtype)
+            covar = torch.tensor(
+                [[[[0.16]]], [[[0.125]]]], device=self.device, dtype=dtype
+            )
+            mvn = MultivariateNormal(mean, covar)
+            p = GPyTorchPosterior(mvn)
+            mm = MockModel(p)
+            weights = torch.tensor([0.5], device=self.device, dtype=dtype)
+            obj = ScalarizedObjective(weights)
+            ei = ExpectedImprovement(model=mm, best_f=0.0, objective=obj)
+            X = torch.rand(2, 1, 2, device=self.device, dtype=dtype)
+            ei_expected = torch.tensor(
+                [[0.2601], [0.1500]], device=self.device, dtype=dtype
+            )
+            torch.allclose(ei(X), ei_expected, atol=1e-4)
+
+            # test objective (multi-output)
+            mean = torch.tensor(
+                [[[-0.25, 0.5]], [[0.2, -0.1]]], device=self.device, dtype=dtype
+            )
+            covar = torch.tensor(
+                [[[0.5, 0.125], [0.125, 0.5]], [[0.25, -0.1], [-0.1, 0.25]]],
+                device=self.device,
+                dtype=dtype,
+            )
+            mvn = MultitaskMultivariateNormal(mean, covar)
+            p = GPyTorchPosterior(mvn)
+            mm = MockModel(p)
+            weights = torch.tensor([2.0, 1.0], device=self.device, dtype=dtype)
+            obj = ScalarizedObjective(weights)
+            ei = ExpectedImprovement(model=mm, best_f=0.0, objective=obj)
+            X = torch.rand(2, 1, 2, device=self.device, dtype=dtype)
+            ei_expected = torch.tensor(
+                [0.6910, 0.5371], device=self.device, dtype=dtype
+            )
+            torch.allclose(ei(X), ei_expected, atol=1e-4)
+
+        # test bad objective class
+        with self.assertRaises(UnsupportedError):
+            ExpectedImprovement(model=mm, best_f=0.0, objective=IdentityMCObjective())
 
 
-class TestPosteriorMean(unittest.TestCase):
-    def test_posterior_mean(self, cuda=False):
-        device = torch.device("cuda") if cuda else torch.device("cpu")
+class TestPosteriorMean(BotorchTestCase):
+    def test_posterior_mean(self):
         for dtype in (torch.float, torch.double):
-            mean = torch.tensor([[0.25]], device=device, dtype=dtype)
+            mean = torch.tensor([[0.25]], device=self.device, dtype=dtype)
             mm = MockModel(MockPosterior(mean=mean))
             module = PosteriorMean(model=mm)
-            X = torch.empty(1, 1, device=device, dtype=dtype)
+            X = torch.empty(1, 1, device=self.device, dtype=dtype)
             pm = module(X)
             self.assertTrue(torch.equal(pm, mean.view(-1)))
             # check for proper error if multi-output model
-            mean2 = torch.rand(1, 2, device=device, dtype=dtype)
+            mean2 = torch.rand(1, 2, device=self.device, dtype=dtype)
             mm2 = MockModel(MockPosterior(mean=mean2))
             module2 = PosteriorMean(model=mm2)
             with self.assertRaises(UnsupportedError):
                 module2(X)
 
-    def test_posterior_mean_cuda(self, cuda=False):
-        if torch.cuda.is_available():
-            self.test_posterior_mean(cuda=True)
-
-    def test_posterior_mean_batch(self, cuda=False):
-        device = torch.device("cuda") if cuda else torch.device("cpu")
+    def test_posterior_mean_batch(self):
         for dtype in (torch.float, torch.double):
-            mean = torch.tensor([-0.5, 0.0, 0.5], device=device, dtype=dtype).view(
+            mean = torch.tensor([-0.5, 0.0, 0.5], device=self.device, dtype=dtype).view(
                 3, 1, 1
             )
             mm = MockModel(MockPosterior(mean=mean))
             module = PosteriorMean(model=mm)
-            X = torch.empty(3, 1, 1, device=device, dtype=dtype)
+            X = torch.empty(3, 1, 1, device=self.device, dtype=dtype)
             pm = module(X)
             self.assertTrue(torch.equal(pm, mean.view(-1)))
             # check for proper error if multi-output model
-            mean2 = torch.rand(3, 1, 2, device=device, dtype=dtype)
+            mean2 = torch.rand(3, 1, 2, device=self.device, dtype=dtype)
             mm2 = MockModel(MockPosterior(mean=mean2))
             module2 = PosteriorMean(model=mm2)
             with self.assertRaises(UnsupportedError):
                 module2(X)
 
-    def test_posterior_mean_batch_cuda(self, cuda=False):
-        if torch.cuda.is_available():
-            self.test_posterior_mean_batch(cuda=True)
 
-
-class TestProbabilityOfImprovement(unittest.TestCase):
-    def test_probability_of_improvement(self, cuda=False):
-        device = torch.device("cuda") if cuda else torch.device("cpu")
+class TestProbabilityOfImprovement(BotorchTestCase):
+    def test_probability_of_improvement(self):
         for dtype in (torch.float, torch.double):
-            mean = torch.tensor([0.0], device=device, dtype=dtype).view(1, 1)
-            variance = torch.ones(1, 1, device=device, dtype=dtype)
+            mean = torch.tensor([0.0], device=self.device, dtype=dtype).view(1, 1)
+            variance = torch.ones(1, 1, device=self.device, dtype=dtype)
             mm = MockModel(MockPosterior(mean=mean, variance=variance))
 
             module = ProbabilityOfImprovement(model=mm, best_f=1.96)
-            X = torch.zeros(1, 1, device=device, dtype=dtype)
+            X = torch.zeros(1, 1, device=self.device, dtype=dtype)
             pi = module(X)
-            pi_expected = torch.tensor(0.0250, device=device, dtype=dtype)
+            pi_expected = torch.tensor(0.0250, device=self.device, dtype=dtype)
             self.assertTrue(torch.allclose(pi, pi_expected, atol=1e-4))
 
             module = ProbabilityOfImprovement(model=mm, best_f=1.96, maximize=False)
-            X = torch.zeros(1, 1, device=device, dtype=dtype)
+            X = torch.zeros(1, 1, device=self.device, dtype=dtype)
             pi = module(X)
-            pi_expected = torch.tensor(0.9750, device=device, dtype=dtype)
+            pi_expected = torch.tensor(0.9750, device=self.device, dtype=dtype)
             self.assertTrue(torch.allclose(pi, pi_expected, atol=1e-4))
 
             # check for proper error if multi-output model
-            mean2 = torch.rand(1, 2, device=device, dtype=dtype)
+            mean2 = torch.rand(1, 2, device=self.device, dtype=dtype)
             variance2 = torch.ones_like(mean2)
             mm2 = MockModel(MockPosterior(mean=mean2, variance=variance2))
             module2 = ProbabilityOfImprovement(model=mm2, best_f=0.0)
             with self.assertRaises(UnsupportedError):
                 module2(X)
 
-    def test_probability_of_improvement_cuda(self, cuda=False):
-        if torch.cuda.is_available():
-            self.test_probability_of_improvement(cuda=True)
-
-    def test_probability_of_improvement_batch(self, cuda=False):
-        device = torch.device("cuda") if cuda else torch.device("cpu")
+    def test_probability_of_improvement_batch(self):
         for dtype in (torch.float, torch.double):
-            mean = torch.tensor([0.0, 0.67449], device=device, dtype=dtype).view(
+            mean = torch.tensor([0.0, 0.67449], device=self.device, dtype=dtype).view(
                 2, 1, 1
             )
             variance = torch.ones_like(mean)
             mm = MockModel(MockPosterior(mean=mean, variance=variance))
             module = ProbabilityOfImprovement(model=mm, best_f=0.0)
-            X = torch.zeros(2, 1, 1, device=device, dtype=dtype)
+            X = torch.zeros(2, 1, 1, device=self.device, dtype=dtype)
             pi = module(X)
-            pi_expected = torch.tensor([0.5, 0.75], device=device, dtype=dtype)
+            pi_expected = torch.tensor([0.5, 0.75], device=self.device, dtype=dtype)
             self.assertTrue(torch.allclose(pi, pi_expected, atol=1e-4))
             # check for proper error if multi-output model
-            mean2 = torch.rand(3, 1, 2, device=device, dtype=dtype)
+            mean2 = torch.rand(3, 1, 2, device=self.device, dtype=dtype)
             variance2 = torch.ones_like(mean2)
             mm2 = MockModel(MockPosterior(mean=mean2, variance=variance2))
             module2 = ProbabilityOfImprovement(model=mm2, best_f=0.0)
             with self.assertRaises(UnsupportedError):
                 module2(X)
 
-    def test_probability_of_improvement_batch_cuda(self, cuda=False):
-        if torch.cuda.is_available():
-            self.test_probability_of_improvement_batch(cuda=True)
 
-
-class TestUpperConfidenceBound(unittest.TestCase):
-    def test_upper_confidence_bound(self, cuda=False):
-        device = torch.device("cuda") if cuda else torch.device("cpu")
+class TestUpperConfidenceBound(BotorchTestCase):
+    def test_upper_confidence_bound(self):
         for dtype in (torch.float, torch.double):
-            mean = torch.tensor([[0.0]], device=device, dtype=dtype)
-            variance = torch.tensor([[1.0]], device=device, dtype=dtype)
+            mean = torch.tensor([[0.0]], device=self.device, dtype=dtype)
+            variance = torch.tensor([[1.0]], device=self.device, dtype=dtype)
             mm = MockModel(MockPosterior(mean=mean, variance=variance))
 
             module = UpperConfidenceBound(model=mm, beta=1.0)
-            X = torch.zeros(1, 1, device=device, dtype=dtype)
+            X = torch.zeros(1, 1, device=self.device, dtype=dtype)
             ucb = module(X)
-            ucb_expected = torch.tensor([1.0], device=device, dtype=dtype)
+            ucb_expected = torch.tensor([1.0], device=self.device, dtype=dtype)
             self.assertTrue(torch.allclose(ucb, ucb_expected, atol=1e-4))
 
             module = UpperConfidenceBound(model=mm, beta=1.0, maximize=False)
-            X = torch.zeros(1, 1, device=device, dtype=dtype)
+            X = torch.zeros(1, 1, device=self.device, dtype=dtype)
             ucb = module(X)
-            ucb_expected = torch.tensor([-1.0], device=device, dtype=dtype)
+            ucb_expected = torch.tensor([-1.0], device=self.device, dtype=dtype)
             self.assertTrue(torch.allclose(ucb, ucb_expected, atol=1e-4))
 
             # check for proper error if multi-output model
-            mean2 = torch.rand(1, 2, device=device, dtype=dtype)
-            variance2 = torch.rand(1, 2, device=device, dtype=dtype)
+            mean2 = torch.rand(1, 2, device=self.device, dtype=dtype)
+            variance2 = torch.rand(1, 2, device=self.device, dtype=dtype)
             mm2 = MockModel(MockPosterior(mean=mean2, variance=variance2))
             module2 = UpperConfidenceBound(model=mm2, beta=1.0)
             with self.assertRaises(UnsupportedError):
                 module2(X)
 
-    def test_upper_confidence_bound_cuda(self, cuda=False):
-        if torch.cuda.is_available():
-            self.test_upper_confidence_bound(cuda=True)
-
-    def test_upper_confidence_bound_batch(self, cuda=False):
-        device = torch.device("cuda") if cuda else torch.device("cpu")
+    def test_upper_confidence_bound_batch(self):
         for dtype in (torch.float, torch.double):
-            mean = torch.tensor([0.0, 0.5], device=device, dtype=dtype).view(2, 1, 1)
-            variance = torch.tensor([1.0, 4.0], device=device, dtype=dtype).view(
+            mean = torch.tensor([0.0, 0.5], device=self.device, dtype=dtype).view(
+                2, 1, 1
+            )
+            variance = torch.tensor([1.0, 4.0], device=self.device, dtype=dtype).view(
                 2, 1, 1
             )
             mm = MockModel(MockPosterior(mean=mean, variance=variance))
             module = UpperConfidenceBound(model=mm, beta=1.0)
-            X = torch.zeros(2, 1, 1, device=device, dtype=dtype)
+            X = torch.zeros(2, 1, 1, device=self.device, dtype=dtype)
             ucb = module(X)
-            ucb_expected = torch.tensor([1.0, 2.5], device=device, dtype=dtype)
+            ucb_expected = torch.tensor([1.0, 2.5], device=self.device, dtype=dtype)
             self.assertTrue(torch.allclose(ucb, ucb_expected, atol=1e-4))
             # check for proper error if multi-output model
-            mean2 = torch.rand(3, 1, 2, device=device, dtype=dtype)
-            variance2 = torch.rand(3, 1, 2, device=device, dtype=dtype)
+            mean2 = torch.rand(3, 1, 2, device=self.device, dtype=dtype)
+            variance2 = torch.rand(3, 1, 2, device=self.device, dtype=dtype)
             mm2 = MockModel(MockPosterior(mean=mean2, variance=variance2))
             module2 = UpperConfidenceBound(model=mm2, beta=1.0)
             with self.assertRaises(UnsupportedError):
                 module2(X)
 
-    def test_upper_confidence_bound_batch_cuda(self, cuda=False):
-        if torch.cuda.is_available():
-            self.test_upper_confidence_bound_batch(cuda=True)
 
-
-class TestConstrainedExpectedImprovement(unittest.TestCase):
-    def test_constrained_expected_improvement(self, cuda=False):
-        device = torch.device("cuda") if cuda else torch.device("cpu")
+class TestConstrainedExpectedImprovement(BotorchTestCase):
+    def test_constrained_expected_improvement(self):
         for dtype in (torch.float, torch.double):
             # one constraint
-            mean = torch.tensor([[-0.5, 0.0]], device=device, dtype=dtype).unsqueeze(
+            mean = torch.tensor(
+                [[-0.5, 0.0]], device=self.device, dtype=dtype
+            ).unsqueeze(dim=-2)
+            variance = torch.ones(1, 2, device=self.device, dtype=dtype).unsqueeze(
                 dim=-2
             )
-            variance = torch.ones(1, 2, device=device, dtype=dtype).unsqueeze(dim=-2)
             mm = MockModel(MockPosterior(mean=mean, variance=variance))
             module = ConstrainedExpectedImprovement(
                 model=mm, best_f=0.0, objective_index=0, constraints={1: [None, 0]}
             )
-            X = torch.empty(1, 1, device=device, dtype=dtype)  # dummy
+            X = torch.empty(1, 1, device=self.device, dtype=dtype)  # dummy
             ei = module(X)
             ei_expected_unconstrained = torch.tensor(
-                0.19780, device=device, dtype=dtype
+                0.19780, device=self.device, dtype=dtype
             )
             ei_expected = ei_expected_unconstrained * 0.5
             self.assertTrue(torch.allclose(ei, ei_expected, atol=1e-4))
@@ -286,9 +334,11 @@ class TestConstrainedExpectedImprovement(unittest.TestCase):
             N = torch.distributions.Normal(loc=0.0, scale=1.0)
             a = N.icdf(torch.tensor(0.75))  # get a so that P(-a <= N <= a) = 0.5
             mean = torch.tensor(
-                [[-0.5, 0.0, 5.0, 0.0]], device=device, dtype=dtype
+                [[-0.5, 0.0, 5.0, 0.0]], device=self.device, dtype=dtype
             ).unsqueeze(dim=-2)
-            variance = torch.ones(1, 4, device=device, dtype=dtype).unsqueeze(dim=-2)
+            variance = torch.ones(1, 4, device=self.device, dtype=dtype).unsqueeze(
+                dim=-2
+            )
             mm = MockModel(MockPosterior(mean=mean, variance=variance))
             module = ConstrainedExpectedImprovement(
                 model=mm,
@@ -296,10 +346,10 @@ class TestConstrainedExpectedImprovement(unittest.TestCase):
                 objective_index=0,
                 constraints={1: [None, 0], 2: [5.0, None], 3: [-a, a]},
             )
-            X = torch.empty(1, 1, device=device, dtype=dtype)  # dummy
+            X = torch.empty(1, 1, device=self.device, dtype=dtype)  # dummy
             ei = module(X)
             ei_expected_unconstrained = torch.tensor(
-                0.19780, device=device, dtype=dtype
+                0.19780, device=self.device, dtype=dtype
             )
             ei_expected = ei_expected_unconstrained * 0.5 * 0.5 * 0.5
             self.assertTrue(torch.allclose(ei, ei_expected, atol=1e-4))
@@ -313,7 +363,7 @@ class TestConstrainedExpectedImprovement(unittest.TestCase):
             )
             ei_min = module_min(X)
             ei_expected_unconstrained_min = torch.tensor(
-                0.6978, device=device, dtype=dtype
+                0.6978, device=self.device, dtype=dtype
             )
             ei_expected_min = ei_expected_unconstrained_min * 0.5
             self.assertTrue(torch.allclose(ei_min, ei_expected_min, atol=1e-4))
@@ -326,19 +376,16 @@ class TestConstrainedExpectedImprovement(unittest.TestCase):
                     constraints={1: [1.0, -1.0]},
                 )
 
-    def test_constrained_expected_improvement_cuda(self, cuda=False):
-        if torch.cuda.is_available():
-            self.test_constrained_expected_improvement(cuda=True)
-
-    def test_constrained_expected_improvement_batch(self, cuda=False):
-        device = torch.device("cuda") if cuda else torch.device("cpu")
+    def test_constrained_expected_improvement_batch(self):
         for dtype in (torch.float, torch.double):
             mean = torch.tensor(
                 [[-0.5, 0.0, 5.0, 0.0], [0.0, 0.0, 5.0, 0.0], [0.5, 0.0, 5.0, 0.0]],
-                device=device,
+                device=self.device,
                 dtype=dtype,
             ).unsqueeze(dim=-2)
-            variance = torch.ones(3, 4, device=device, dtype=dtype).unsqueeze(dim=-2)
+            variance = torch.ones(3, 4, device=self.device, dtype=dtype).unsqueeze(
+                dim=-2
+            )
             N = torch.distributions.Normal(loc=0.0, scale=1.0)
             a = N.icdf(torch.tensor(0.75))  # get a so that P(-a <= N <= a) = 0.5
             mm = MockModel(MockPosterior(mean=mean, variance=variance))
@@ -348,22 +395,18 @@ class TestConstrainedExpectedImprovement(unittest.TestCase):
                 objective_index=0,
                 constraints={1: [None, 0], 2: [5.0, None], 3: [-a, a]},
             )
-            X = torch.empty(3, 1, 1, device=device, dtype=dtype)  # dummy
+            X = torch.empty(3, 1, 1, device=self.device, dtype=dtype)  # dummy
             ei = module(X)
+            self.assertTrue(ei.shape == torch.Size([3]))
             ei_expected_unconstrained = torch.tensor(
-                [0.19780, 0.39894, 0.69780], device=device, dtype=dtype
+                [0.19780, 0.39894, 0.69780], device=self.device, dtype=dtype
             )
             ei_expected = ei_expected_unconstrained * 0.5 * 0.5 * 0.5
             self.assertTrue(torch.allclose(ei, ei_expected, atol=1e-4))
 
-    def test_constrained_expected_improvement_batch_cuda(self, cuda=False):
-        if torch.cuda.is_available():
-            self.test_constrained_expected_improvement_batch(cuda=True)
 
-
-class TestNoisyExpectedImprovement(unittest.TestCase):
-    def _get_model(self, cuda=False, dtype=torch.float):
-        device = torch.device("cuda") if cuda else torch.device("cpu")
+class TestNoisyExpectedImprovement(BotorchTestCase):
+    def _get_model(self, dtype=torch.float):
         state_dict = {
             "mean_module.constant": torch.tensor([-0.0066]),
             "covar_module.raw_outputscale": torch.tensor(1.0143),
@@ -375,21 +418,22 @@ class TestNoisyExpectedImprovement(unittest.TestCase):
             "covar_module.outputscale_prior.concentration": torch.tensor(2.0),
             "covar_module.outputscale_prior.rate": torch.tensor(0.1500),
         }
-        train_x = torch.linspace(0, 1, 10, device=device, dtype=dtype)
+        train_x = torch.linspace(0, 1, 10, device=self.device, dtype=dtype).unsqueeze(
+            -1
+        )
         train_y = torch.sin(train_x * (2 * math.pi))
-        noise = torch.tensor(NEI_NOISE, device=device, dtype=dtype)
+        noise = torch.tensor(NEI_NOISE, device=self.device, dtype=dtype)
         train_y += noise
         train_yvar = torch.full_like(train_y, 0.25 ** 2)
-        train_x = train_x.view(-1, 1)
         model = FixedNoiseGP(train_X=train_x, train_Y=train_y, train_Yvar=train_yvar)
         model.load_state_dict(state_dict)
         model.to(train_x)
         model.eval()
         return model
 
-    def test_noisy_expected_improvement(self, cuda=False):
+    def test_noisy_expected_improvement(self):
         for dtype in (torch.float, torch.double):
-            model = self._get_model(cuda=cuda, dtype=dtype)
+            model = self._get_model(dtype=dtype)
             X_observed = model.train_inputs[0]
             nEI = NoisyExpectedImprovement(model, X_observed, num_fantasies=5)
             X_test = torch.tensor(
@@ -413,14 +457,10 @@ class TestNoisyExpectedImprovement(unittest.TestCase):
             with torch.no_grad():
                 nEI(X_test)
             # test non-FixedNoiseGP model
-            other_model = SingleTaskGP(X_observed, model.train_targets)
+            other_model = SingleTaskGP(X_observed, model.train_targets.unsqueeze(-1))
             with self.assertRaises(UnsupportedError):
                 NoisyExpectedImprovement(other_model, X_observed, num_fantasies=5)
             # Test with minimize
             nEI = NoisyExpectedImprovement(
                 model, X_observed, num_fantasies=5, maximize=False
             )
-
-    def test_noisy_expected_improvement_cuda(self, cuda=False):
-        if torch.cuda.is_available():
-            self.test_noisy_expected_improvement(cuda=True)
