@@ -3,7 +3,6 @@
 # Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
 
 import math
-import unittest
 import warnings
 
 import torch
@@ -17,6 +16,7 @@ from botorch.models.gp_regression import (
 from botorch.models.utils import add_output_dim
 from botorch.posteriors import GPyTorchPosterior
 from botorch.sampling import SobolQMCNormalSampler
+from botorch.utils.testing import BotorchTestCase
 from gpytorch.kernels import MaternKernel, ScaleKernel
 from gpytorch.likelihoods import (
     FixedNoiseGaussianLikelihood,
@@ -37,12 +37,10 @@ def _get_random_data(batch_shape, num_outputs, n=10, **tkwargs):
         n, num_outputs, **tkwargs
     ).repeat(batch_shape + torch.Size([1, 1]))
 
-    if num_outputs == 1:
-        train_y = train_y.squeeze(-1)
     return train_x, train_y
 
 
-class TestSingleTaskGP(unittest.TestCase):
+class TestSingleTaskGP(BotorchTestCase):
     def _get_model_and_data(self, batch_shape, num_outputs, **tkwargs):
         train_X, train_Y = _get_random_data(
             batch_shape=batch_shape, num_outputs=num_outputs, **tkwargs
@@ -51,12 +49,12 @@ class TestSingleTaskGP(unittest.TestCase):
         model = SingleTaskGP(**model_kwargs)
         return model, model_kwargs
 
-    def test_gp(self, cuda=False):
+    def test_gp(self):
         for batch_shape in (torch.Size(), torch.Size([2])):
             for num_outputs in (1, 2):
                 for double in (False, True):
                     tkwargs = {
-                        "device": torch.device("cuda") if cuda else torch.device("cpu"),
+                        "device": self.device,
                         "dtype": torch.double if double else torch.float,
                     }
                     model, _ = self._get_model_and_data(
@@ -122,16 +120,12 @@ class TestSingleTaskGP(unittest.TestCase):
                         torch.allclose(pvar, pvar_exp, rtol=1e-4, atol=1e-05)
                     )
 
-    def test_gp_cuda(self):
-        if torch.cuda.is_available():
-            self.test_gp(cuda=True)
-
-    def test_condition_on_observations(self, cuda=False):
+    def test_condition_on_observations(self):
         for batch_shape in (torch.Size(), torch.Size([2])):
             for num_outputs in (1, 2):
                 for double in (False, True):
                     tkwargs = {
-                        "device": torch.device("cuda") if cuda else torch.device("cpu"),
+                        "device": self.device,
                         "dtype": torch.double if double else torch.float,
                     }
                     model, model_kwargs = self._get_model_and_data(
@@ -232,16 +226,12 @@ class TestSingleTaskGP(unittest.TestCase):
                                 )
                             )
 
-    def test_condition_on_observations_cuda(self):
-        if torch.cuda.is_available():
-            self.test_condition_on_observations(cuda=True)
-
-    def test_fantasize(self, cuda=False):
+    def test_fantasize(self):
         for batch_shape in (torch.Size(), torch.Size([2])):
             for num_outputs in (1, 2):
                 for double in (False, True):
                     tkwargs = {
-                        "device": torch.device("cuda") if cuda else torch.device("cpu"),
+                        "device": self.device,
                         "dtype": torch.double if double else torch.float,
                     }
                     model, model_kwargs = self._get_model_and_data(
@@ -259,10 +249,6 @@ class TestSingleTaskGP(unittest.TestCase):
                     )
                     self.assertIsInstance(fm, model.__class__)
 
-    def test_fantasize_cuda(self):
-        if torch.cuda.is_available():
-            self.test_fantasize(cuda=True)
-
 
 class TestFixedNoiseGP(TestSingleTaskGP):
     def _get_model_and_data(self, batch_shape, num_outputs, **tkwargs):
@@ -277,12 +263,12 @@ class TestFixedNoiseGP(TestSingleTaskGP):
         model = FixedNoiseGP(**model_kwargs)
         return model, model_kwargs
 
-    def test_fixed_noise_likelihood(self, cuda=False):
+    def test_fixed_noise_likelihood(self):
         for batch_shape in (torch.Size(), torch.Size([2])):
             for num_outputs in (1, 2):
                 for double in (False, True):
                     tkwargs = {
-                        "device": torch.device("cuda") if cuda else torch.device("cpu"),
+                        "device": self.device,
                         "dtype": torch.double if double else torch.float,
                     }
                     model, model_kwargs = self._get_model_and_data(
@@ -298,10 +284,6 @@ class TestFixedNoiseGP(TestSingleTaskGP):
                         )
                     )
 
-    def test_fixed_noise_likelihood_cuda(self):
-        if torch.cuda.is_available():
-            self.test_fixed_noise_likelihood(cuda=True)
-
 
 class TestHeteroskedasticSingleTaskGP(TestSingleTaskGP):
     def _get_model_and_data(self, batch_shape, num_outputs, **tkwargs):
@@ -317,12 +299,12 @@ class TestHeteroskedasticSingleTaskGP(TestSingleTaskGP):
         model = HeteroskedasticSingleTaskGP(**model_kwargs)
         return model, model_kwargs
 
-    def test_heteroskedastic_likelihood(self, cuda=False):
+    def test_heteroskedastic_likelihood(self):
         for batch_shape in (torch.Size(), torch.Size([2])):
             for num_outputs in (1, 2):
                 for double in (False, True):
                     tkwargs = {
-                        "device": torch.device("cuda") if cuda else torch.device("cpu"),
+                        "device": self.device,
                         "dtype": torch.double if double else torch.float,
                     }
                     model, _ = self._get_model_and_data(
@@ -337,24 +319,23 @@ class TestHeteroskedasticSingleTaskGP(TestSingleTaskGP):
                         model.likelihood.noise_covar.noise_model, SingleTaskGP
                     )
 
-    def test_heteroskedastic_likelihood_cuda(self):
-        if torch.cuda.is_available():
-            self.test_heteroskedastic_likelihood(cuda=True)
-
-    def test_condition_on_observations(self, cuda=False):
+    def test_condition_on_observations(self):
         with self.assertRaises(NotImplementedError):
-            super().test_condition_on_observations(cuda=cuda)
+            super().test_condition_on_observations()
 
-    def test_fantasize(self, cuda=False):
+    def test_fantasize(self):
         with self.assertRaises(NotImplementedError):
-            super().test_fantasize(cuda=cuda)
+            super().test_fantasize()
 
 
 def _get_pvar_expected(posterior, model, X, num_outputs):
+    lh_kwargs = {}
+    if isinstance(model.likelihood, FixedNoiseGaussianLikelihood):
+        lh_kwargs["noise"] = model.likelihood.noise.mean().expand(X.shape[:-1])
     if num_outputs == 1:
-        return model.likelihood(posterior.mvn, X).variance.unsqueeze(-1)
+        return model.likelihood(posterior.mvn, X, **lh_kwargs).variance.unsqueeze(-1)
     X_, odi = add_output_dim(X=X, original_batch_shape=model._input_batch_shape)
-    pvar_exp = model.likelihood(model(X_), X_).variance
+    pvar_exp = model.likelihood(model(X_), X_, **lh_kwargs).variance
     return torch.stack(
         [pvar_exp.select(dim=odi, index=i) for i in range(num_outputs)], dim=-1
     )
