@@ -2,9 +2,8 @@
 
 # Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
 
-import unittest
-
 from botorch.models.model import Model
+from botorch.utils.testing import BotorchTestCase
 
 
 class NotSoAbstractBaseModel(Model):
@@ -12,7 +11,7 @@ class NotSoAbstractBaseModel(Model):
         pass
 
 
-class TestBaseModel(unittest.TestCase):
+class TestBaseModel(BotorchTestCase):
     def test_abstract_base_model(self):
         with self.assertRaises(TypeError):
             Model()
