@@ -5,7 +5,7 @@
 import torch
 from botorch.acquisition import qExpectedImprovement
 from botorch.gen import gen_candidates_scipy, get_best_candidates
-from gpytorch import settings
+from gpytorch import settings as gpt_settings
 
 from ..test_gen import TestBaseCandidateGeneration
 
@@ -14,10 +14,10 @@ EPS = 1e-8
 
 
 class TestRandomRestartOptimization(TestBaseCandidateGeneration):
-    def test_random_restart_optimization(self, cuda=False):
+    def test_random_restart_optimization(self):
         for double in (True, False):
-            self._setUp(double=double, cuda=cuda)
-            with settings.debug(False):
+            self._setUp(double=double)
+            with gpt_settings.debug(False):
                 best_f = self.model(self.train_x).mean.max().item()
             qEI = qExpectedImprovement(self.model, best_f=best_f)
             bounds = torch.tensor([[0.0], [1.0]]).type_as(self.train_x)
@@ -33,7 +33,3 @@ class TestRandomRestartOptimization(TestBaseCandidateGeneration):
                 batch_candidates=batch_candidates, batch_values=batch_acq_values
             )
             self.assertTrue(-EPS <= candidates <= 1 + EPS)
-
-    def test_random_restart_optimization_cuda(self):
-        if torch.cuda.is_available():
-            self.test_random_restart_optimization(cuda=True)
