@@ -2,9 +2,8 @@
 
 # Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
 
-import unittest
-
 from botorch.posteriors import Posterior
+from botorch.utils.testing import BotorchTestCase
 
 
 class NotSoAbstractPosterior(Posterior):
@@ -24,7 +23,7 @@ class NotSoAbstractPosterior(Posterior):
         pass
 
 
-class TestPosterior(unittest.TestCase):
+class TestPosterior(BotorchTestCase):
     def test_abstract_base_posterior(self):
         with self.assertRaises(TypeError):
             Posterior()
