@@ -2,10 +2,10 @@
 
 # Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
 
-import unittest
 
 import torch
 from botorch.utils import apply_constraints, get_objective_weights_transform
+from botorch.utils.testing import BotorchTestCase
 from torch import Tensor
 
 
@@ -21,7 +21,7 @@ def minus_one_f(samples: Tensor) -> Tensor:
     return -torch.ones(samples.shape[0:-1], device=samples.device, dtype=samples.dtype)
 
 
-class TestApplyConstraints(unittest.TestCase):
+class TestApplyConstraints(BotorchTestCase):
     def test_apply_constraints(self):
         # nonnegative objective, one constraint
         samples = torch.randn(1)
@@ -61,7 +61,7 @@ class TestApplyConstraints(unittest.TestCase):
             )
 
 
-class TestGetObjectiveWeightsTransform(unittest.TestCase):
+class TestGetObjectiveWeightsTransform(BotorchTestCase):
     def test_NoWeights(self):
         Y = torch.ones(5, 2, 4, 1)
         objective_transform = get_objective_weights_transform(None)
